import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class RestSDK {
	private static String AccountSid="";
	private static String AccountToken="";
	private static String AppId="";
	private static String ServerIP="";
	private static String ServerPort="8883";
	private static String SoftVersion="2013-12-26";
	private static String timestamp="";

	public void init(String accountsSid, String accountToken,String appId, String serverIP) {
		AccountSid=accountsSid;
		AccountToken=accountToken;
		AppId=appId;
		ServerIP=serverIP;
	}
    /**
     * 向指定 URL 发送POST方法的请求
     * 
     * @param url
     *            发送请求的 URL
     * @param param
     *            请求参数
     * @return 所代表远程资源的响应结果
     */
    public static String sendPost(String url, String param) {
        PrintWriter out = null;
        BufferedReader in = null;
        String result = "";
        try {

            //生成auth
    		String src = AccountSid + ":" + timestamp;
    		String auth = new sun.misc.BASE64Encoder().encode( src.getBytes() ); 
            URL realUrl = new URL(url);
            System.out.println(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();

            // 设置通用的请求属性
            conn.setRequestProperty("accept", "application/xml");
            conn.setRequestProperty("connection", "close");
            conn.setRequestProperty("content-type","application/xml;charset=utf-8");
            conn.setRequestProperty("authorization",auth);
            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 获取URLConnection对象对应的输出流
            out = new PrintWriter(conn.getOutputStream());
            // 发送请求参数
            out.print(param);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(
                    new InputStreamReader(conn.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            System.out.println("发送 POST 请求出现异常！"+e);
            e.printStackTrace();
        }
        //使用finally块来关闭输出流、输入流
        finally{
            try{
                if(out!=null){
                    out.close();
                }
                if(in!=null){
                    in.close();
                }
            }
            catch(IOException ex){
                ex.printStackTrace();
            }
        }
        return result;
    }   
    
    /**
     * 创建队列
     * @param friendlyName 队列类型
     * @param typedes 队列类型描述
     * @param worktime 队列工作时间
     * @param offworkprompt 非工作时间队列提示音
     * @param offworkdate 非工作日期
     * @param offworkweekday 每周的非工作时间
     */
    public static String createqueue(String queuetype,String  typedes,String worktime,String offworkprompt,String  offworkdate,String offworkweekday){
    	//获取系统时间
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");//设置日期格式
        timestamp=df.format(new Date());
		//生成sig
        String signature="";
		String sig = AccountSid + AccountToken + timestamp;
		try {
			signature = md5Digest(sig);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		//拼接请求包体
		String requsetbody = "";

		requsetbody="<?xml version='1.0' encoding='utf-8'?><Request><Appid>"+AppId+"</Appid><CreateQueue queuetype='"+queuetype+"' typedes='"+typedes+"' worktime='"+worktime+"' offworkprompt='"+offworkprompt+"' offworkdate='"+offworkdate+"' offworkweekday='"+offworkweekday+"'/></Request>";
		String url= "https://"+ServerIP+":"+ServerPort+"/"+SoftVersion+"/Accounts/"+AccountSid+"/ivr/createqueue?sig="+signature;
		return sendPost(url,requsetbody);
    	
    }
    
    /**
     * 座席上班
     * @param number 座席号码
     * @param agentid 座席ID
     * @param agenttype 座席所在队列
     * @param agentstate 座席状态
     */
    public static String agentonwork(String number,String  agentid,String agenttype,String agentstate){
    	//获取系统时间
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");//设置日期格式
        timestamp=df.format(new Date());
		//生成sig
        String signature="";
		String sig = AccountSid + AccountToken + timestamp;
		try {
			signature = md5Digest(sig);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		//拼接请求包体
		String requsetbody = "";

		requsetbody="<?xml version='1.0' encoding='utf-8'?><Request><Appid>"+AppId+"</Appid><AgentOnWork number='"+number+"' agentid='"+agentid+"' agenttype='"+agenttype+"' agentstate='"+agentstate+"'/></Request>";
		String url= "https://"+ServerIP+":"+ServerPort+"/"+SoftVersion+"/Accounts/"+AccountSid+"/ivr/agentonwork?sig="+signature;
		return sendPost(url,requsetbody);
    	
    }
    
    /**
     * 座席准备就绪
     * @param agentid 座席id
     * @param action 就绪失败的回调url
     * @param state true表示状态更新为准备就绪，false表示状态更新为准备中
     * @param priority 是否优先接听客户电话
     * @param force 是否强制设置状态
     */
    public static String agentready(String agentid,String  action,String state,String priority,String force){
    	//获取系统时间
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");//设置日期格式
        timestamp=df.format(new Date());
		//生成sig
        String signature="";
		String sig = AccountSid + AccountToken + timestamp;
		try {
			signature = md5Digest(sig);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		//拼接请求包体
		String requsetbody = "";

		requsetbody="<?xml version='1.0' encoding='utf-8'?><Request><Appid>"+AppId+"</Appid><AgentReady agentid='"+agentid+"' action='"+action+"' state='"+state+"' priority='"+priority+"' force='"+force+"'/></Request>";
		String url= "https://"+ServerIP+":"+ServerPort+"/"+SoftVersion+"/Accounts/"+AccountSid+"/ivr/agentready?sig="+signature;
		return sendPost(url,requsetbody);
    	
    }
    
	/**
     * IVR外呼
     * @param number   待呼叫号码，为Dial节点的属性
     * @param userdata 用户数据，在<startservice>通知中返回，只允许填写数字字符，为Dial节点的属性
     * @param record   是否录音，可填项为true和false，默认值为false不录音，为Dial节点的属性
     */
    public static String Dial(String number,String  userdata,String record){
    	//获取系统时间
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");//设置日期格式
        timestamp=df.format(new Date());
		//生成sig
        String signature="";
		String sig = AccountSid + AccountToken + timestamp;
		try {
			signature = md5Digest(sig);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		//拼接请求包体
		String requsetbody = "";

		requsetbody="<?xml version='1.0' encoding='utf-8'?><Request><Appid>"+AppId+"</Appid><Dial number='"+number+"' userdata='"+userdata+"' record='"+record+"'/></Request>";
		String url= "https://"+ServerIP+":"+ServerPort+"/"+SoftVersion+"/Accounts/"+AccountSid+"/ivr/dial?sig="+signature;
		return sendPost(url,requsetbody);
    	
    }
    
    //MD5加密
    public static String md5Digest(String src) throws NoSuchAlgorithmException, UnsupportedEncodingException
    {
      MessageDigest md = MessageDigest.getInstance("MD5");
      byte[] b = md.digest(src.getBytes("utf-8"));
      return byte2HexStr(b);
    }
    private static String byte2HexStr(byte[] b)
    {
      StringBuilder sb = new StringBuilder();
      for (int i = 0; i < b.length; ++i) {
        String s = Integer.toHexString(b[i] & 0xFF);
        if (s.length() == 1)
          sb.append("0");

        sb.append(s.toUpperCase());
      }
      return sb.toString();
    }
}
